import React, { useEffect, useState } from 'react'
import BaseTmp from '@/components/BaseTmp'
import api from '@/services/api/index'
import { PlayCircleTwoTone, PauseCircleTwoTone } from '@ant-design/icons'
import { Row, Tabs, Button, Select, Input, Col, Table, Tooltip, Divider, Switch, message } from 'antd';
import AudioManage from '@/common/AudioManage';
import { useStateSync } from '@/hooks/index' 
import { history } from 'umi';
import { descText } from '@/common/MessageManage';

const { TabPane } = Tabs

const {
  getQuestionBank,
  getQuestionCard,
  getQuestionBook,
  useQuestionCard,
  delQuestionCard
} = api.question

interface ISelectTmp {
  dataList?: Record<string, any>[],
  onChange?: (value: string) => void
}

/** 搜索模板 */
const SelectTmp: React.FC<ISelectTmp> = ({
  dataList,
  onChange
}) => {

  return  <Select allowClear style={{ width: 120 }} onChange={onChange} >
            {
              dataList?.map( data => <Select.Option value={data.id} key={data.id} >{data.label}</Select.Option>)
            }
          </Select>
}

/** 表格 */
const QuestionTable = ({
  loading,
  dataSource,
  onChangePagination,
  pagination,
  onChangeSwitch,
  onDel,
  onEdit
}: {
  loading: boolean,
  dataSource: any[],
  pagination: IPagination
  onChangePagination: (current: number) => void,
  onChangeSwitch: (check: boolean, state: any) => void,
  onDel: (record: any) => void,
  onEdit: (record: any) => void
}) => {

  const [ playingVoice, setPlayingVoice ] = useState<string>()

  const playAudio = (voiceUrl: string) => {
    AudioManage.play(voiceUrl)
    setPlayingVoice(voiceUrl)
  }

  const closeAudio = () => {
    AudioManage.pause()
    setPlayingVoice('')
  }

  const columns = [
    {
      title: '题卡',
      dataIndex: 'title',
      key: 'title',
    },
    {
      title: '问题',
      dataIndex: 'label',
      key: 'label',
      ellipsis: {
        showTitle: false,
      },
      render: (label: string) => <Tooltip   placement="topLeft" title={label} > { label }  </Tooltip>
    },
    {
      title: '音频',
      dataIndex: 'voiceUrl',
      key: 'voiceUrl',
      render: ( voiceUrl: string ) => (
        <>
          <span style={{cursor: "pointer"}} >
            {
              playingVoice === voiceUrl ?  
                <PauseCircleTwoTone onClick={closeAudio} style={{fontSize: '30px'}} /> 
                :  
                <PlayCircleTwoTone onClick={() => playAudio(voiceUrl)} style={{fontSize: '30px'}} /> 
            }
          </span>
        </>
      )
    },
    {
      title: '是否启用',
      dataIndex: 'state',
      key: 'state',
      render: (state, record) => 
      <Switch 
        checked={record.state === 1} 
        checkedChildren="已启用" 
        unCheckedChildren="已禁用" 
        onChange={(check) => onChangeSwitch(check, record)} 
      />
    },
    {
      title: '创建时间',
      dataIndex: 'createTime',
      key: 'createTime',
    },
    {
      title: '操作',
      dataIndex: 'action',
      key: 'action',
      render: (action, records: any) => (
        <>
          {/* <a >查看</a>
          <Divider type="vertical" /> */}
          <a onClick={() => onEdit(records) } >编辑</a>
          <Divider type="vertical" />
          <a  onClick={onDel} >删除</a>
        </>
      )
    }
  ];

  return <Table 
    loading={loading}
    style={{marginTop: '20px'}} 
    dataSource={dataSource}   
    columns={columns} 
    pagination={{
      showSizeChanger: false,
      position: ["bottomRight"],
      total: pagination?.total,
      current: pagination?.current, 
      onChange: onChangePagination
    }}
   />;
}

interface ITabsState {
  id: string,
  label: string
}

interface IQuestionSearch {
  bookList: any[],
  onChangeBook: (id: string) => void,
  onChangeUseState: (state: number) => void,
  onSearch: () => void,
  addQuestionCard: () => void,
  onInputLabel: (e: InputEvent) => void
}

const useStateList = [
  {label: '启用', id: 1, state: 1},
  {label: '禁用', id: -1, state: -1},
]

/** 搜索 */
const QuestionSearch: React.FC<IQuestionSearch> = ({
  bookList,
  onChangeBook,
  onChangeUseState,
  onSearch,
  addQuestionCard,
  onInputLabel
}) => {

  return <Row>
    {/* <Col span={4}> <span>学段筛选： </span> <SelectTmp  dataList={bookList} onChange={onChange}  /> </Col> */}
    <Col span={4}> <span>教材筛选： </span> <SelectTmp  dataList={bookList} onChange={onChangeBook} /></Col>
    <Col span={4}> <span>启用状态： </span> <SelectTmp  dataList={useStateList} onChange={onChangeUseState}  /></Col>
    <Col span={3}> <Input placeholder='可通过题干搜索' style={{width: '150px'}} onInput={onInputLabel} /> </Col>
    <Col span={3}> <Button type="primary" onClick={onSearch} >搜索</Button> </Col>
    {/* <Col span={2}> <Button type="primary" danger  >批量删除</Button></Col> */}
    {/* <Col span={2}> <Button type="primary" color='' >加入比赛</Button></Col> */}
    <Col span={2}> <Button type="primary" onClick={addQuestionCard} > 添加题卡 </Button> </Col>
  </Row>
}

interface IQuestionTabs {
  children: React.ReactChild,
  onChangeTabs: (e: string) => void
  tabsList: ITabsState[] 
}

interface IPagination {
  total: number,
  current: number,
  state: number | string,
  textbookId: string
}

/** tabs  */
const QuestionTabs: React.FC<IQuestionTabs> = ({
  children,
  onChangeTabs,
  tabsList
}) => {

  return  <Tabs defaultActiveKey="1" onChange={(e) => onChangeTabs(e)}>
              {
                tabsList.map( item => (
                  <TabPane tab={item.label} key={item.id}>
                    { children }
                  </TabPane>
                ))
              }
          </Tabs>
}

const initPagination = {
  total: 0,
  current: 1,
  state: '',
  textbookId: ''
}

const QuestionCard: React.FC = () => {

  /**TabsData */
  const [ tabsList, setTabsList ] = useState<ITabsState[]>([])
  
  /**current tabs key */
  const [ currentTabsKey,  setCurrentTabsKey ] = useState<string>('')

  /**分页 */
  const [ pagination, setPgination ] =  useStateSync<IPagination>(initPagination)

  const [ loading, setLoading ] = useState<boolean>(false)

  /**数据 */
  const [ dataSource, setDataSource ] = useState<Record<string, any>[]>([])
 
  /**教材数据 */
  const [ bookList, setBookList ] = useState([])

  /**current table item */
  const [ currentDataSource, setCurrentDataSource ] = useState()

  const [ label, setLabel ] = useState<String>('')

  useEffect( () => {
    _getQuestionBank()
  }, [])

  useEffect( () => {

    _getQuestionBook()
    
    setPgination(initPagination)

  }, [currentTabsKey])

  useEffect(() => {

    _getQuestionCard()

  }, [ pagination.current, currentTabsKey])

  /**删除题卡  */
   const _delQuestionCard = async (records: any) => {
     const { status, result } = await delQuestionCard(records.id)
     if ( status === 200 ) {
       message.success( descText.delTextSuccess )
       _getQuestionCard()
     }
   }

   /**编辑题卡 --> 携带题卡的cardId 跳转到下一添加题卡页面 */
  const onEdit = (records: any) => {
    history.push(`/QuestionBankManage/opration-card?cardId=${records.id}`)
  }

  /**添加题卡 --> 跳转到添加题卡页面 */
  const addQuestionCard = () => {
    history.push('/QuestionBankManage/opration-card')
  }

  /**上下架状态 */
  const onChangeSwitch = (check: boolean, records: any) => {
    _useQuestionCard(check, records)
  }

  /**search */
  const onSearch = () => _getQuestionCard()

  /**search label  */ 
  const onInputLabel = (e) => setLabel(e.target.value)

  /**query condition book */
  const onChangeBook = (id: string) => {
    console.log("uery condition book:", id);
    
    setPgination({
      ...pagination,
      textbookId: id
    })
  }

  /**query condition useState */
  const onChangeUseState = (state: number) => {
    console.log('onChangeUseState:', state);
    setPgination({
      ...pagination,
      state
    })
  }

  /**分页 */
  const onChangePagination = (current: number) => {
    setPgination({
      ...pagination,
      current: current
    })
  }

  /** rquest 启用或者停用题卡 */
  const _useQuestionCard = async (check: boolean, records: any) => {
    console.log(check, 'check');
    const state = check ? 1 : -1
    
    const { status, result } = await useQuestionCard(records.id , {state: state})
    if ( status === 200 && result ) {
        message.success(state ===  1 ? '启用成功' : '禁用成功')
        const $r = dataSource.map( ds =>  {
          return {
            ...ds,
            state: records.id === ds.id ? state :  ds.state
          }
        })
        setDataSource( $r)
        // _getQuestionCard()
    }
  }

  /**获取教材 */
  const _getQuestionBook = async () => {
    const { status, result } = await getQuestionBook({bankId: currentTabsKey})
    if (status === 200) { 
      setBookList(result)
    }
  }

  /**获取题卡 */
  const _getQuestionCard = async (_pagination?: IPagination) => {

    setLoading(true)
    console.log(currentTabsKey, 'pagination');
    
    const { status, result } = await getQuestionCard(_pagination? { 
      bankId: currentTabsKey,
        ..._pagination
      } : {
      bankId: currentTabsKey,
      page: pagination?.current, 
      pageSize: 10,
      state: pagination.state,
      label,
      textbookId: pagination.textbookId
    })
    setLoading(false)
    if ( status === 200 ) {
      console.log(result.records, 'result');
      setDataSource(result.records)
      setPgination({
        ...pagination,
        total: result.total,
        current: result.current
      })
    }
  }

  const _getQuestionBank = async () => {
    const { status, result } =  await getQuestionBank()
    
    if (status === 200) {
      setTabsList(result)
      setCurrentTabsKey(result[0].id)
    }

  }

  const onChangeTabs = (e: string) => {
    console.log(e, 'onChangeTabs');
    setCurrentTabsKey(e)
  }

  return  <BaseTmp>
            <QuestionTabs  
              onChangeTabs={onChangeTabs}
              tabsList={tabsList}
            >
              <>
                <QuestionSearch 
                  bookList={bookList}
                  addQuestionCard={addQuestionCard}
                  onSearch={onSearch}
                  onInputLabel={onInputLabel}
                  onChangeBook={onChangeBook}
                  onChangeUseState={onChangeUseState}
                />
                <QuestionTable 
                  loading={loading}
                  pagination={pagination}
                  dataSource={dataSource}
                  onDel={_delQuestionCard}
                  onEdit={onEdit}
                  onChangeSwitch={onChangeSwitch}
                  onChangePagination={onChangePagination}
                />
              </>
            </QuestionTabs>
          </BaseTmp>
}

export default QuestionCard


